"""
Custom fields for storing RDF primitives.

Based on http://blog.elsdoerfer.name/2008/01/08/fuzzydates-or-one-django-model-field-multiple-database-columns/
"""
from django.db import models
from rdflib.graph import Graph
from rdflib.term import BNode, URIRef, Literal


class LiteralField(models.TextField):
    """
    Custom field for storing literals.
    """

    __metaclass__ = models.SubfieldBase
    description = "Field for storing Literals, including their type and language"

    def to_python(self, value):
        if not value:
            return None

        if isinstance(value, Literal):
            return value

        parts = value.split('^^')
        if len(parts) != 3:
            raise ValueError("Wrong value: {0}".format(value))
        return Literal(parts[0], parts[1] or None, parts[2] or None)

    def value_to_string(self, obj):
        value = self._get_val_from_obj(obj)
        return self.get_prep_value(value)

    def get_prep_value(self, value):
        if not isinstance(value, Literal):
            raise TypeError("Value {0} has the wrong type: {1}".format(value, value.__class__))

        return unicode(value) + "^^" + (value.language or '') + "^^" + (value.datatype or '')


def deserialize_uri(value):
    """
    Deserialize a representation of a BNode or URIRef.
    """
    if isinstance(value, BNode):
        return value
    if isinstance(value, URIRef):
        return value
    if not value:
        return None
    if not isinstance(value, basestring):
        raise ValueError("Cannot create URI from {0} of type {1}".format(value, value.__class__))
    if value.startswith("_:"):
        return BNode(value[2:])
    return URIRef(value)


def serialize_uri(value):
    """
    Serialize a BNode or URIRef.
    """
    if isinstance(value, BNode):
        return value.n3()
    if isinstance(value, URIRef):
        return unicode(value)
    raise ValueError("Cannot get prepvalue for {0} of type {1}".format(value, value.__class__))


class URIField(models.CharField):
    """
    Custom field for storing URIRefs and BNodes.

    URIRefs are stored as themselves; BNodes are stored in their Notation3 serialization.
    """

    __metaclass__ = models.SubfieldBase
    description = "Field for storing URIRefs and BNodes."

    def __init__(self, *args, **kwargs):
        if not 'max_length' in kwargs:
            kwargs['max_length'] = 500
        super(URIField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        return deserialize_uri(value)

    def value_to_string(self, obj):
        return serialize_uri(self._get_val_from_obj(obj))

    def get_prep_value(self, value):
        return serialize_uri(value)


class GraphReferenceField(models.CharField):
    """
    Custom field for storing graph references.
    """

    __metaclass__ = models.SubfieldBase
    description = "Field for storing references to Graphs"

    def to_python(self, value):
        if isinstance(value, Graph):
            return value.identifier

        return deserialize_uri(value)

    def value_to_string(self, obj):
        value = self._get_val_from_obj(obj)
        return self.get_prep_value(value)

    def get_prep_value(self, value):
        if isinstance(value, Graph):
            return serialize_uri(value.identifier)

        return serialize_uri(value)



"""
Settings for testing the application.
"""
import os

DEBUG = True

DJANGO_RDFLIB_DEVELOP = True

DB_PATH = os.path.abspath(os.path.join(__file__, '..', '..', '..', 'rdflib_django.db'))

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': DB_PATH,
        'USER': '',
        'PASSWORD': '',
        'HOST': '',
        'PORT': '',
        }
}

SITE_ID = 1

STATIC_URL = '/static/'

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'django.contrib.admindocs',

    'rdflib_django',
    )
ROOT_URLCONF = 'rdflib_django.urls'

LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'formatters': {
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
        },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'simple'
        },
        },
    'loggers': {
        '': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'INFO',
            },
        }
}



"""
Management command for exporting RDF from the store.
"""
from optparse import make_option
from django.core.management.base import BaseCommand
import sys
from rdflib.term import URIRef
from rdflib_django import utils


class Command(BaseCommand):
    """
    Command object for exporting RDF.
    """

    option_list = BaseCommand.option_list + (
        make_option('--context', '-c', type='string', dest='context',
            help='Only RDF data from the context with this identifier will be exported. If not specified, a new blank ' +
                 'context is created.'),

        make_option('--store', '-s', type='string', dest='store',
            help='RDF data will be exported from the store with this identifier. If not specified, the default store ' +
                 'is used.'),

        make_option('--format', '-f', type='string', dest='format', default='xml',
            help='Format of the RDF data. This option accepts all formats allowed by rdflib. Defaults to xml.')
    )

    help = """Exports an RDF resource.

Examples:
    {0} rdf_export my_file.rdf
    {0} rdf_export --format n3 my_file.n3
    {0} rdf_export --context http://example.com/context
    """.format(sys.argv[0])
    args = 'file'

    def handle(self, *args, **options):
        store_id = options.get('store')
        context_id = options.get('context')
        target = args[0] if args else sys.stdout

        if context_id:
            graph = utils.get_named_graph(URIRef(context_id), store_id=store_id)
        else:
            graph = utils.get_conjunctive_graph(store_id)

        #noinspection PyUnresolvedReferences
        graph.serialize(target, format=options.get('format'))



"""
Base forms for editing the models in this module. You can use or extend these forms in your
project to ensure that all validation is correct.
"""
from django import forms
from rdflib_django import models
from rdflib import namespace


class NamespaceForm(forms.ModelForm):
    """
    Form for editing namespaces.
    """

    class Meta:
        model = models.NamespaceModel
        fields = ('prefix', 'uri')

    def __init__(self, *args, **kwargs):
        super(NamespaceForm, self).__init__(*args, **kwargs)

        if self.instance.fixed:
            self.fields['prefix'].widget.attrs['readonly'] = True
            self.fields['uri'].widget.attrs['readonly'] = True

    def clean_prefix(self):
        """
        Validates the prefix
        """
        if self.instance.fixed:
            return self.instance.prefix

        prefix = self.cleaned_data['prefix']
        if not namespace.is_ncname(prefix):
            raise forms.ValidationError("This is an invalid prefix")

        return prefix

    def clean_uri(self):
        """
        Validates the URI
        """
        if self.instance.fixed:
            return self.instance.uri

        uri = self.cleaned_data['uri']
        # todo: URI validation
        return uri



"""
Defines admin options for this RDFlib implementation.
"""
from django.contrib import admin
from rdflib_django import models, forms


class NamedGraphAdmin(admin.ModelAdmin):
    """
    Admin module for named graphs.
    """

    list_display = ('identifier', )
    ordering = ('identifier', )
    search_fields = ('identifier', )


class NamespaceAdmin(admin.ModelAdmin):
    """
    Admin module for managing namespaces.
    """
    list_display = ('prefix', 'uri', 'fixed')
    ordering = ('-fixed', 'prefix')
    search_fields = ('prefix', 'uri')
    form = forms.NamespaceForm

    def get_actions(self, request):
        return []

    def has_delete_permission(self, request, obj=None):
        """
        Default namespaces cannot be deleted.
        """
        if obj is not None and obj.fixed:
            return False

        return super(NamespaceAdmin, self).has_delete_permission(request, obj)

admin.site.register(models.NamedGraph, NamedGraphAdmin)
admin.site.register(models.NamespaceModel, NamespaceAdmin)

admin.site.register(models.URIStatement)
admin.site.register(models.LiteralStatement)



"""
The rdflib_django implementation uses Django models to store its triples.

The underlying models are Resource centric, because rdflib-django is intended
to be used for publishing resources.
"""
from django.db import models
from django.utils.translation import ugettext as _
from django_extensions.db.fields import UUIDField
from rdflib_django import fields


class NamedGraph(models.Model):
    """
    Models a context which represents a named graph.
    """

    identifier = fields.URIField(verbose_name=_("Identifier"), unique=True)

    class Meta:
        verbose_name = _("named graph")
        verbose_name_plural = _("named graphs")

    def __unicode__(self):
        return u"{0}".format(self.identifier, "identifier")


class NamespaceModel(models.Model):
    """
    A namespace definition.

    In essence, a namespace consists of a prefix and a URI. However, the namespaces in rdflib_django
    also have an extra field called '`fixed`' - this is used to mark namespaces that cannot be
    remapped such as ``xml``, ``rdf`` and ``rdfs``.
    """

    prefix = models.CharField(max_length=50, verbose_name=_("Prefix"), unique=True)
    uri = models.CharField(max_length=500, verbose_name=_("URI"), db_index=True, unique=True)
    fixed = models.BooleanField(verbose_name=_("Fixed"), editable=False, default=False)

    class Meta:
        verbose_name = _("namespace")
        verbose_name_plural = _("namespaces")

    def __unicode__(self):
        return "@prefix {0}: <{1}>".format(self.prefix, self.uri)


class URIStatement(models.Model):
    """
    Statement where the object is a URI.
    """

    id = UUIDField("ID", primary_key=True)
    subject = fields.URIField(verbose_name=_("Subject"), db_index=True)
    predicate = fields.URIField(_("Predicate"), db_index=True)
    object = fields.URIField(_("Object"), db_index=True)
    context = models.ForeignKey(NamedGraph, verbose_name=_("Context"))

    class Meta:
        unique_together = ('subject', 'predicate', 'object', 'context')

    def __unicode__(self):
        return u"{0}, {1}".format(self.as_triple(), self.context.identifier)    # pylint: disable=E1101

    def as_triple(self):
        """
        Converts this predicate to a triple.
        """
        return self.subject, self.predicate, self.object


class LiteralStatement(models.Model):
    """
    Statement where the object is a literal.
    """

    id = UUIDField("ID", primary_key=True)
    subject = fields.URIField(verbose_name=_("Subject"), db_index=True)
    predicate = fields.URIField(_("Predicate"), db_index=True)
    object = fields.LiteralField(_("Object"), db_index=True)
    context = models.ForeignKey(NamedGraph, verbose_name=_("Context"))

    class Meta:
        unique_together = ('subject', 'predicate', 'object', 'context')

    def __unicode__(self):
        return u"{0}, {1}".format(self.as_triple(), self.context.identifier)    # pylint: disable=E1101

    def as_triple(self):
        """
        Converts this predicate to a triple.
        """
        return self.subject, self.predicate, self.object



"""
Unittests and doctests for the rdflib_django app.
"""
import doctest
from django.utils import unittest
import rdflib_django
from rdflib_django import store, test_store, test_rdflib, test_seq, test_namespaces


def suite():
    """
    Generate the test suite.
    """
    s = unittest.TestSuite()
    s.addTest(doctest.DocTestSuite(rdflib_django))
    s.addTest(doctest.DocTestSuite(store))
    s.addTest(unittest.findTestCases(test_store))
    s.addTest(unittest.findTestCases(test_rdflib))
    s.addTest(unittest.findTestCases(test_seq))
    s.addTest(unittest.findTestCases(test_namespaces))
    return s



"""
Essential implementation of the Store interface defined by RDF lib.
"""
from django.db.utils import IntegrityError
import rdflib
from rdflib.store import VALID_STORE
from rdflib.term import Literal, Identifier
from rdflib_django import models
from rdflib_django.models import NamespaceModel


DEFAULT_STORE = "Default Store"

DEFAULT_NAMESPACES = (
    ("xml", u"http://www.w3.org/XML/1998/namespace"),
    ("rdf", u"http://www.w3.org/1999/02/22-rdf-syntax-ns#"),
    ("rdfs", u"http://www.w3.org/2000/01/rdf-schema#")
    )


def _get_query_sets_for_object(o):
    """
    Determines the correct query set based on the object.

    If the object is a literal, it will return a query set over LiteralStatements.
    If the object is a URIRef or BNode, it will return a query set over Statements.
    If the object is unknown, it will return both the LiteralStatement and Statement query sets.

    This method always returns a list of size at least one.
    """
    if o:
        if isinstance(o, Literal):
            query_sets = [models.LiteralStatement.objects]
        else:
            query_sets = [models.URIStatement.objects]
    else:
        query_sets = [models.URIStatement.objects, models.LiteralStatement.objects]
    return query_sets


def _get_named_graph(context):
    """
    Returns the named graph for this context.
    """
    if context is None:
        return None

    return models.NamedGraph.objects.get_or_create(identifier=context.identifier)[0]


class DjangoStore(rdflib.store.Store):
    """
    RDFlib Store implementation the uses Django Models for storage and retrieval.

    >>> g = rdflib.Graph('Django')

    The implementation is context aware, and uses Django transactions.

    >>> g.store.context_aware
    True
    >>> g.store.transaction_aware
    False

    The implementation does not support formula's.

    >>> g.store.formula_aware
    False

    The implementation provides a single store with the identifier DEFAULT_STORE. This store
    is always present and needs not be opened.

    >>> g.store.identifier
    'Default Store'

    Using other stores is not allowed

    >>> g = DjangoStore(identifier='HelloWorld')
    Traceback (most recent call last):
        ...
    ValueError: multiple stores are not allowed


    """

    context_aware = True
    formula_aware = False
    transaction_aware = False

    def __init__(self, configuration=None, identifier=DEFAULT_STORE):
        if identifier and identifier != DEFAULT_STORE:
            raise ValueError("multiple stores are not allowed")

        self.identifier = DEFAULT_STORE
        super(DjangoStore, self).__init__(configuration, identifier)
        self.open()

    def open(self, configuration=None, create=False):
        """
        Opens the underlying store. This is only necessary when opening
        a store with another identifier than the default identifier.

        >>> g = rdflib.Graph('Django')
        >>> g.open(configuration=None, create=False) == rdflib.store.VALID_STORE
        True
        """
        return VALID_STORE

    def destroy(self, configuration=None):
        """
        Completely destroys a store and all the contexts and triples in the store.

        >>> store = DjangoStore()
        >>> g = rdflib.Graph(store=store)
        >>> g.open(configuration=None, create=True) == rdflib.store.VALID_STORE
        True
        >>> g.open(configuration=None, create=False) == rdflib.store.VALID_STORE
        True
        >>> g.destroy(configuration=None)
        >>> g.open(configuration=None, create=False) == rdflib.store.VALID_STORE
        True
        """
        models.NamedGraph.objects.all().delete()
        models.URIStatement.objects.all().delete()
        models.LiteralStatement.objects.all().delete()

    def add(self, (s, p, o), context, quoted=False):
        """
        Adds a triple to the store.

        >>> from rdflib.term import URIRef
        >>> from rdflib.namespace import RDF

        >>> subject = URIRef('http://zoowizard.org/resource/Artis')
        >>> object = URIRef('http://schema.org/Zoo')
        >>> g = rdflib.Graph('Django')
        >>> g.add((subject, RDF.type, object))
        >>> len(g)
        1

        """
        assert isinstance(s, Identifier)
        assert isinstance(p, Identifier)
        assert isinstance(o, Identifier)
        assert not quoted

        named_graph = _get_named_graph(context)

        query_set = _get_query_sets_for_object(o)[0]
        query_set.get_or_create(
            subject=s,
            predicate=p,
            object=o,
            context=named_graph,
            )

    def remove(self, (s, p, o), context=None):
        """
        Removes a triple from the store.
        """
        named_graph = _get_named_graph(context)
        query_sets = _get_query_sets_for_object(o)

        filter_parameters = dict()
        if named_graph is not None:
            filter_parameters['context_id'] = named_graph.id
        if s:
            filter_parameters['subject'] = s
        if p:
            filter_parameters['predicate'] = p
        if o:
            filter_parameters['object'] = o

        query_sets = [qs.filter(**filter_parameters) for qs in query_sets]  # pylint: disable=W0142

        for qs in query_sets:
            qs.delete()

    def triples(self, (s, p, o), context=None):
        """
        Returns all triples in the current store.
        """
        named_graph = _get_named_graph(context)
        query_sets = _get_query_sets_for_object(o)

        filter_parameters = dict()
        if named_graph is not None:
            filter_parameters['context_id'] = named_graph.id
        if s:
            filter_parameters['subject'] = s
        if p:
            filter_parameters['predicate'] = p
        if o:
            filter_parameters['object'] = o

        query_sets = [qs.filter(**filter_parameters) for qs in query_sets]  # pylint: disable=W0142

        for qs in query_sets:
            for statement in qs:
                triple = statement.as_triple()
                yield triple, context

    def __len__(self, context=None):
        """
        Returns the number of statements in this Graph.
        """
        named_graph = _get_named_graph(context)
        if named_graph is not None:
            return (models.LiteralStatement.objects.filter(context_id=named_graph.id).count()
                    + models.URIStatement.objects.filter(context_id=named_graph.id).count())
        else:
            return (models.URIStatement.objects.values('subject', 'predicate', 'object').distinct().count()
                    + models.LiteralStatement.objects.values('subject', 'predicate', 'object').distinct().count())

    ####################
    # CONTEXT MANAGEMENT

    def contexts(self, triple=None):
        for c in models.NamedGraph.objects.all():
            yield c.identifier

    ######################
    # NAMESPACE MANAGEMENT

    def bind(self, prefix, namespace):
        for ns in DEFAULT_NAMESPACES:
            if ns[0] == prefix or unicode(ns[1]) == unicode(namespace):
                return

        try:
            ns = NamespaceModel(prefix=prefix, uri=namespace)
            ns.save()
        except IntegrityError:
            NamespaceModel.objects.filter(prefix=prefix).delete()
            NamespaceModel.objects.filter(uri=namespace).delete()
            NamespaceModel(prefix=prefix, uri=namespace).save()

    def prefix(self, namespace):
        try:
            ns = NamespaceModel.objects.get(uri=namespace)
            return ns.prefix
        except NamespaceModel.DoesNotExist:
            return None

    def namespace(self, prefix):
        try:
            ns = NamespaceModel.objects.get(prefix=prefix)
            return ns.uri
        except NamespaceModel.DoesNotExist:
            return None

    def namespaces(self):
        for ns in NamespaceModel.objects.all():
            yield ns.prefix, ns.uri



"""
Unittests for namespace management.
"""
from django.utils import unittest
from rdflib.graph import Graph
from rdflib.term import URIRef
from rdflib_django.store import DjangoStore


XML_NAMESPACE = ("xml", URIRef("http://www.w3.org/XML/1998/namespace"))
RDF_NAMESPACE = ("rdf", URIRef("http://www.w3.org/1999/02/22-rdf-syntax-ns#"))
RDFS_NAMESPACE = ("rdfs", URIRef("http://www.w3.org/2000/01/rdf-schema#"))


class NamespaceTest(unittest.TestCase):
    """
    Tests for the weird behavior of RDFlib graphs and the default
    namespaces.
    """

    def setUp(self):
        self.store = DjangoStore()

    def testDefaultNamespaces(self):
        """
        Test the presence of the default namespaces.
        """
        g = Graph(store=self.store)
        namespaces = list(g.namespaces())
        self.assertIn(XML_NAMESPACE, namespaces)
        self.assertIn(RDF_NAMESPACE, namespaces)
        self.assertIn(RDFS_NAMESPACE, namespaces)

        # and again; by default, initializing the graph will re-bind the namespaces
        # using a naive implementation, this will fail.
        g = Graph(store=self.store)
        g_namespaces = list(g.namespaces())
        self.assertIn(XML_NAMESPACE, g_namespaces)
        self.assertIn(RDF_NAMESPACE, g_namespaces)
        self.assertIn(RDFS_NAMESPACE, g_namespaces)

    def testCannotUpdateDefaultNamespaces(self):
        """
        Binding the prefix OR the URI of a default namespaces is a no-op.
        """
        g = Graph(store=self.store)
        self.assertIn(XML_NAMESPACE, list(g.namespaces()))

        g.bind("hello-world", XML_NAMESPACE[1])
        self.assertIn(XML_NAMESPACE, list(g.namespaces()))

        g.bind(XML_NAMESPACE, URIRef("http://example.com/xml"))
        self.assertIn(XML_NAMESPACE, list(g.namespaces()))

    def testBindingNamespaces(self):
        """
        Binding custom namespaces just works.
        """
        g = Graph(store=self.store)
        ns = ("prefix", URIRef("http://example.com/prefix"))

        self.assertNotIn(ns, list(g.namespaces()))

        g.bind(ns[0], ns[1])
        self.assertIn(ns, list(g.namespaces()))



"""
Management command for parsing RDF into the store.
"""
from optparse import make_option
from django.core.management.base import BaseCommand, CommandError
import sys
from django.db import transaction
from rdflib.graph import Graph
from rdflib.term import URIRef, BNode
from rdflib_django import utils


class Command(BaseCommand):
    """
    Command object for importing RDF.
    """

    option_list = BaseCommand.option_list + (
        make_option('--store', '-s', type='string', dest='store',
            help='RDF data will be imported into the store with this identifier. If not specified, the default store ' +
                 'is used.'),

        make_option('--context', '-c', type='string', dest='context',
            help='RDF data will be imported into a context with this identifier. If not specified, a new blank ' +
                 'context is created.'),

        make_option('--format', '-f', type='string', dest='format', default='xml',
            help='Format of the RDF data. This option accepts all formats allowed by rdflib. Defaults to xml.')
    )

    help = """Imports an RDF resource.

Examples:
    {0} rdf_import my_file.rdf
    {0} rdf_import --format n3 my_file.n3
    {0} rdf_import --context http://zoowizard.eu http://zoowizard.eu/datasource/zoochat/294
    """.format(sys.argv[0])
    args = 'file-or-resource'

    @transaction.commit_on_success
    def handle(self, *args, **options):
        if not args:
            raise CommandError("No file or resource specified.")

        info = options.get('verbosity') >= 2
        store_id = options.get('store')
        context_id = options.get('context')
        source = args[0]

        if info:
            print("Parsing {0}".format(source))

        intermediate = Graph()
        try:
            intermediate.parse(source, format=options.get('format'))
        except Exception as e:
            raise CommandError(e)

        if info:
            print("Parsed {0} triples".format(len(intermediate)))

        identifier = URIRef(context_id) if context_id else BNode()
        graph = utils.get_named_graph(identifier, store_id=store_id)

        if info:
            print("Storing {0} triples".format(len(intermediate)))
        for triple in intermediate:
            graph.add(triple)
        if info:
            print("Done")



"""
The application provides no URLs of its own.
In development mode, this will include the admin package.
"""
from django.conf import settings
from django.conf.urls import patterns, include

if hasattr(settings, 'DJANGO_RDFLIB_DEVELOP') and getattr(settings, 'DJANGO_RDFLIB_DEVELOP'):
    from django.contrib import admin
    admin.autodiscover()

    urlpatterns = patterns('',
        (r'^admin/doc/', include('django.contrib.admindocs.urls')),
        (r'^admin/', include(admin.site.urls)),
    )



"""
Unit tests for the store class. Includes all unit tests that are hard or annoying to doctest.
"""
import datetime
from django import test
import rdflib
from rdflib.graph import Graph
from rdflib.namespace import RDF, RDFS, Namespace
from rdflib.term import URIRef, Literal, BNode


EX = Namespace("http://www.example.com/")


artis = URIRef('http://zoowizard.eu/resource/Artis')
berlin_zoo = URIRef('http://zoowizard.eu/resource/Berlin_Zoo')
zoo = URIRef('http://schema.org/Zoo')
org = URIRef('http://schema.org/Organisation')
anonymous = BNode()
artis_label = Literal('Artis')
date_literal = Literal(datetime.date.today())
number_literal = Literal(14)
bool_literal = Literal(True)
graph_context = Graph(identifier=EX['graph-context'])


class GraphTest(test.TestCase):
    """
    Several checks on the store by using it through the official Graph interface.
    """

    def setUp(self):
        self.graph = rdflib.Graph('Django')

    def test_add_uri_statement(self):
        """
        What happens if we add statements that are all URI's
        """
        self.graph.add((artis, RDF.type, zoo))
        self.assertEquals(len(self.graph), 1)

        self.graph.add((artis, RDF.type, org))
        self.assertEquals(len(self.graph), 2)

        self.graph.add((artis, RDF.type, zoo))
        self.assertEquals(len(self.graph), 2)

    def test_single_triple(self):
        """
        Returning the triples should give the correct result
        """
        self.graph.add((artis, RDF.type, zoo))
        triples = list(self.graph.triples((None, None, None)))
        self.assertEquals(len(triples), 1)

        self.assertTupleEqual(triples[0], (artis, RDF.type, zoo))

    def test_multiple_triples(self):
        """
        Returning the triples should give the correct result
        """
        self.graph.add((artis, RDF.type, zoo))
        self.graph.add((artis, RDF.type, org))
        self.graph.add((berlin_zoo, RDF.type, zoo))
        self.assertEquals(len(list(self.graph.triples((None, None, None)))), 3)

        self.assertEquals(len(list(self.graph.triples((artis, None, None)))), 2)
        self.assertEquals(len(list(self.graph.triples((None, RDF.type, None)))), 3)
        self.assertEquals(len(list(self.graph.triples((None, None, zoo)))), 2)
        self.assertEquals(len(list(self.graph.triples((None, None, org)))), 1)

    def test_blank_nodes(self):
        """
        Adding and querying for blank nodes should also work.
        """
        self.graph.add((artis, RDFS.seeAlso, anonymous))
        self.graph.add((anonymous, RDF.type, zoo))

        triple = list(self.graph.triples((None, None, zoo)))[0]
        self.assertTupleEqual(triple, (anonymous, RDF.type, zoo))

    def test_literals(self):
        """
        Adding and querying dates should also work.
        """
        self.graph.add((artis, RDFS.label, artis_label))
        self.graph.add((artis, EX['date'], date_literal))
        self.graph.add((artis, EX['bool'], bool_literal))
        self.graph.add((artis, EX['number'], number_literal))
        self.assertEquals(len(self.graph), 4)

        self.assertEquals(self.graph.value(artis, RDFS.label), artis_label)
        self.assertEquals(self.graph.value(artis, EX['date']), date_literal)
        self.assertEquals(self.graph.value(artis, EX['bool']), bool_literal)
        self.assertEquals(self.graph.value(artis, EX['number']), number_literal)



import os
from setuptools import setup, find_packages

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name = "rdflib-django",
    version = "0.3",
    url = "http://github.com/publysher/rdflib-django",
    license = 'MIT',
    description = "Store implementation for RDFlib using Django models as its backend",
    long_description = read('README.rst'),
    keywords = 'django rdf rdflib store', 

    author = 'Yigal Duppen',
    author_email = 'yigal@publysher.nl',

    packages = find_packages('src'),
    package_dir = {'': 'src'},
    zip_safe = True,

    install_requires = ['rdflib>=3.2.1'],

    classifiers = [
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



"""
Utility functions for using rdflib_django.
"""
from rdflib.graph import ConjunctiveGraph, Graph
from rdflib.store import VALID_STORE
from rdflib.term import URIRef
from rdflib_django.store import DjangoStore, DEFAULT_STORE


def get_conjunctive_graph(store_id=None):
    """
    Returns an open conjunctive graph.
    """
    if not store_id:
        store_id = DEFAULT_STORE

    store = DjangoStore(DEFAULT_STORE)
    graph = ConjunctiveGraph(store=store, identifier=store_id)
    if graph.open(None) != VALID_STORE:
        raise ValueError("The store identified by {0} is not a valid store".format(store_id))
    return graph


def get_named_graph(identifier, store_id=DEFAULT_STORE, create=True):
    """
    Returns an open named graph.
    """
    if not isinstance(identifier, URIRef):
        identifier = URIRef(identifier)

    store = DjangoStore(store_id)
    graph = Graph(store, identifier=identifier)
    if graph.open(None, create=create) != VALID_STORE:
        raise ValueError("The store identified by {0} is not a valid store".format(store_id))
    return graph
